// HTML4 / HTML5: No differences
define(["jquery"], function (jq) {

	"use strict";

	/**
	 * @constructor
	 */
	return function (context, metadata) {
		var hoverElement = null,
			duration,
			size,
			m_sImage,
			m_nWidth,
			m_nHeight,
			canFireTouchend = "ontouchend" in document,
			canFireTouchstart = "ontouchstart" in document,
			supportsPointerEvents = window && window.PointerEvent;

		// sets private variables to initial parameter values
		function initializeParameters(parameters) {
			duration = parameters.asNumber("speed", "400");
			size = parameters.asNumber("size", "40");
			m_nWidth = parameters.asNumber("wrapperWidth", "300");

			return getImageParameters(parameters.get("picture", null), m_nWidth);
		}

		function createMarkup() {
			var cssWidth = m_nWidth + "px",
				cssHeight = m_nHeight + "px",
				objShell = jq(context.getElement())
					.width(cssWidth)
					.height(cssHeight);

			// if we are in athoring mode, create an object shield so we are able to select and drag element
			if (context.stage.isInAuthoringMode()) {
				objShell.append(
					jq(document.createElement("div"))
						.css({
							"position": "absolute",
							"left": 0,
							"background-color": "#fff",
							"opacity": "0.4",
							"width": cssWidth,
							"height": cssHeight,
							"z-index": "9999"
						})
				);
			}

			hoverElement = jq(document.createElement("div"))
				.attr("id", "thumbnails_" + metadata.getId())
				.width(cssWidth)
				.height(cssHeight)
				.append(
					jq(document.createElement("img"))
						.attr("src", m_sImage)
						.width(cssWidth)
						.height(cssHeight)
						.css({
							"-webkit-touch-callout": "none"
						})
						.on("touchmove", function(e) {e.preventDefault ? e.preventDefault() : true;})
				);

			objShell.append(hoverElement);
		}

		function lockNavigation() {
			if(context.stage.trigger !== undefined) {
				context.stage.trigger.lock(metadata.getId());
			}
		}

		function unlockNavigation() {
			if(context.stage.trigger !== undefined) {
				context.stage.trigger.unlock(metadata.getId());
			}
		}

		function addHoverEffect(options) {
			if(!hoverElement) {
				return;
			}
			hoverElement.find("img:first").each(function(i, img){
				var jqImage = jq(img),
					theDefaults = {
						size: 20,
						duration: 200,
						zIndexActive: 100,
						zIndexNormal: 1
					},
					opts = jq.extend({}, theDefaults, options);

				// parent must be relatively positioned
				jqImage
					.parent()
					.css({
						position: "relative"
					});

				// pulsing element must be absolutely positioned
				jqImage.css({
					"position": "absolute",
					"top": 0,
					"left": 0
				});

				jqImage.data("hoverEffect.size", {
					w: jqImage.width(),
					h: jqImage.height()
				});

				// On mouse over / touch finger
				jqImage.on(canFireTouchstart ? "touchstart" : (supportsPointerEvents ? "pointerenter" : "mouseenter"), function () {
					var jqElem = jq(this),
						sizeImage = jqElem.data("hoverEffect.size"),
						deltaWidth = opts.size,
						w = sizeImage.w,
						h = sizeImage.h,
						aspectRatio = w / (h ? h : 1),
						deltaHeight = deltaWidth / aspectRatio;

					jqElem
						.parent()
						.css({
							"z-index": opts.zIndexActive
						});

					jqElem
						.stop()
						.animate({
							top: ((deltaHeight / -2.0) + "px"),
							left: ((deltaWidth / -2.0) + "px"),
							height: ((h + deltaHeight) + "px"),
							width: ((w + deltaWidth) + "px")
						}, opts.duration, "swing");
				});

				// On mouse out / release finger
				jqImage.on(canFireTouchend ? "touchend" : (supportsPointerEvents ? "pointerleave" : "mouseleave"), function() {
					var jqElem = jq(this),
						size = jqElem.data("hoverEffect.size"),
						w = size.w,
						h = size.h;

					jqElem
						.stop()
						.animate({
							"top": 0,
							"left": 0,
							"height": (h + "px"),
							"width": (w + "px")
						}, opts.duration, "swing", function () {
							jqElem
								.parent()
								.css("z-index", opts.zIndexNormal);
						});
				});
			});
		}

		function getImageParameters(image, width) {
			var deferred = jq.Deferred(),
				promise = deferred.promise(),
				defaultImage = new Image();

			defaultImage.onload = function() {
				var result = {
						fileName: "",
						width: width,
						height: 0
					},
					imgWidth,
					imgHeight;
				try {
					result.fileName = this.src;
					result.height = width / this.width * this.height;

					if(typeof image !== "object" || image.fileName === undefined || image.width === undefined || image.height === undefined) {
						return;
					}

					imgWidth = 0.0 || parseFloat(image.width);
					imgHeight = 0.0 || parseFloat(image.height);
					result.height = 0.0;
					result.fileName = context.stage.getSharedPath() + image.fileName;
					if(imgWidth !== 0) {
						result.height = (width / imgWidth) * imgHeight;
					}
				} finally {
					deferred.resolve(result);
				}
			};
			defaultImage.src = context.getPath() + "hoverpulse.jpg";

			return promise;
		}

		// revealing public API
		return {
			onLoading: function () {
				context.page.event.bindOnEnter(this);
				context.page.event.bindOnExit(this);

				initializeParameters(metadata.parameters).then(function (imageParams) {
					m_sImage = imageParams.fileName;
					m_nHeight = imageParams.height;

					createMarkup();

					if (!context.stage.isInAuthoringMode()) {
						addHoverEffect({
							// number of pixels to pulse element (in each direction)
							size: size,
							// duration of the animation
							duration: duration
						});
					}

					jq(context.getElement())
						.on(canFireTouchstart ? "touchstart" : (supportsPointerEvents ? "pointerdown" : "mousedown"), lockNavigation)
						.on(canFireTouchend ? "touchleave touchcancel touchend" : (supportsPointerEvents ? "pointerout pointerup" : "mouseout mouseup"), unlockNavigation);
				});
			},

			onEnterPage: function () {
			},

			onExitPage: unlockNavigation,

			onAllComponentsReady: function () {
				addHoverEffect({
					// number of pixels to pulse element (in each direction)
					size: size,
					// duration of the animation
					duration: duration
				});
			}
		};
	};
});