// HTML4 / HTML5: No differences
define(["jquery","css!./component.css"], function (jq) {

	"use strict";

	/**
	 * @constructor
	 */
	return function (context, metadata) {

	    var videoElement = null,
            controlType = metadata.parameters.get("video_control", "html5"),
            userAgent = navigator.userAgent.toLowerCase(),
            isIPad = userAgent.indexOf("ipad") !== -1,
        	isIPhone = userAgent.indexOf("iphone") !== -1,
        	isAndroid = userAgent.indexOf("android") !== -1,
	        canFireTouchstart = "ontouchstart" in document,
			canFireTouchend = "ontouchend" in document,
			canFireTouchmove = "ontouchmove" in document,
			canFireTouchcancel = "ontouchcancel" in document,
			supportsPointerEvents = window && window.PointerEvent,
        	startVideoEvent = canFireTouchstart ? "touchstart" : (supportsPointerEvents ? "pointerdown" : "mousedown"),
            stopVideoEvent = canFireTouchend && canFireTouchcancel ? "touchend touchcancel" : (supportsPointerEvents ? "pointerup" : "mouseup"),
            moveEvent = canFireTouchmove ? "touchmove" : (supportsPointerEvents ? "pointermove mousemove" : "mousemove"),
            leaveEvent = canFireTouchend ? "touchend" : (supportsPointerEvents ? "pointerout" : "mouseout"),
	        randomizedUrlPostfix = Math.floor(Math.random() * 999999);

        function createVideo() {

			var videoFileUrl = getFileUrl("video_file"),
				shellElement = jq(context.getElement());

            shellElement.css({
                "width": metadata.getWidth() + "px",
                "height": metadata.getHeight() + "px"
            });

            if( videoFileUrl.length === 0 ){
				return;
			}

			videoElement = createVideoElement();
            shellElement.append(videoElement);
            loadVideo( videoFileUrl );
			setAudioState();

			if(isAndroid) {
                // Create a layer on Android devices so that the user
                // can not start playback as long as not enough content is loaded:
                shellElement.append( createPlaybackShield() );
			}

            if( controlType === "none") {
                getController().play();
            }
		}

        function createVideoElement() {

            var videoElement = jq(document.createElement("video"))
                .addClass("sc-video")
                .attr("preload", "metadata");

            addListeners( videoElement );

            if (canFireTouchmove) {
                videoElement.on("touchmove", preventVerticalScrolling);
            }

            videoElement.on(moveEvent, function (e) {
                e.originalEvent.stopPropagation ? e.originalEvent.stopPropagation() : true;
                typeof e.originalEvent.cancelBubble !== "undefined" ? e.originalEvent.cancelBubble = true : true;
            });

            var posterFileUrl = getFileUrl("poster_file");
            if( posterFileUrl.length > 0 ){
                videoElement.attr("poster", posterFileUrl );
            }

            if (isAndroid) {
                videoElement.attr("preload", "auto");
                videoElement.on("canplay", function() {
                    jq("#video" + randomizedUrlPostfix).remove();
                });
            }

            switch( controlType ) {
                case "none":
                    videoElement.attr("autoplay", "true");
                    break;
                case "html5":
                default:
                    videoElement.attr("controls", "true");
            }

            return videoElement;
        }

        function loadVideo( videoFileUrl ) {
            videoElement.attr("src", videoFileUrl);
        }

        function createPlaybackShield(){
            return jq(document.createElement("div"))
                .css({
                    "position": "absolute",
                    "background-color": "#cccccc",
                    "opacity": 0.1,
                    "width": "100%",
                    "height": "100%"
                })
                .attr("id", "video" + randomizedUrlPostfix);
        }

        function getController() {
            var video = videoElement.get(0);
            return (video && video.controller !== undefined && video.controller !== null) ? video.controller : video;
        }

        function getFileUrl( paramName ) {

            var file = metadata.parameters.get( paramName, "" ),
                fileName,
				fileUrl;

            if( !file || typeof file !== "object" ) {
                return "";
            }

            fileName = file.fileName;
            if( !fileName || fileName === "" ) {
                return "";
            }

            fileUrl = context.stage.getSharedPath() + fileName;

            if (isAndroid) {
                fileUrl += "?" + randomizedUrlPostfix;
            }

            return fileUrl;
        }

        function setAudioState() {
			videoElement.prop("muted", !context.stage.audio.isOn() );
		}

		function preventVerticalScrolling(e) {
			var result = isOnShadowedElement(e);
			if (result.onSlider || result.onTimeline) {
				lockNavigation();
				return true;
			}
			e.originalEvent.preventDefault ? e.originalEvent.preventDefault() : true;
		}

		function switchPlayMode() {
			if (!videoElement || videoElement.length <= 0) {
				return;
			}

			var controller = getController();
			if (!controller) {
				return;
			}

			switch( controlType ) {
				case "none":
					if (controller.paused === false) {
						controller.pause();
					}
					else {
                        controller.play();
					}
					break;
				case "html5":
				default:
					if (isAndroid) {
						// Show or hide controls on android devices:
						videoElement.removeAttr("controls");
						videoElement.attr("controls", "controls");
					}
			}
		}

		function prepareForAuthoringMode() {
			if (!context.stage.isInAuthoringMode()) {
				return false;
			}
			jq(context.getElement())
				.text("Video")
				.css({
					"border": "1px solid #4d4d4d",
					"font-family": "Helvetica, Arial, sans-serif",
					"background-color": "#cccccc",
					"font-size": "14px",
					"text-align": "center",
					"padding": 0
				});
			return true;
		}

		function removeVideo() {
			unlockNavigation();

			if( !videoElement || videoElement.length === 0 ) {
				return;
			}

			videoElement.each( function() {
				this.pause();
				delete this;
			});
			videoElement.remove();
			videoElement = null;
		}

		function lockNavigation() {
			if (context.stage.isInAuthoringMode() || !context.stage.trigger) {
				return;
			}
			context.stage.trigger.lock(metadata.getId());
		}

		function unlockNavigation() {
			if (context.stage.isInAuthoringMode() || !context.stage.trigger) {
				return;
			}
			context.stage.trigger.unlock(metadata.getId());
		}

		function isOnShadowedElement(e) {
			if (e && e.originalEvent && e.originalEvent.changedTouches && e.originalEvent.changedTouches.length > 0) {
				var changedTouches = e.originalEvent.changedTouches[0],
					pseudoAttribute = changedTouches.target.getAttribute("pseudo"),
					pseudoAttributeLower;
				if (typeof pseudoAttribute === "string") {
					pseudoAttributeLower = pseudoAttribute.toLowerCase();
					if (pseudoAttributeLower.indexOf("webkit-media") !== -1) {
						// Don't lockNavigation if touched on the timeline slider
						return {
							onTimeline: (pseudoAttributeLower.indexOf("timeline") !== -1),
							onSlider: (pseudoAttributeLower.indexOf("slider") !== -1),
							onStartButton: (pseudoAttributeLower.indexOf("start-playback") !== -1),
							onWebkitMedia: true
						};
					}
				}
			}
			return {
				onTimeline: false,
				onSlider: false,
				onStartButton: false,
				onWebkitMedia: false
			};
		}

		function cancelEvent(e) {
			if (e && e.originalEvent.preventDefault) {
				e.originalEvent.preventDefault();
			}
			if (e && e.originalEvent.stopPropagation()) {
				e.originalEvent.stopPropagation();
			}
			return false;
		}

        function addListeners( videoElement ) {
            videoElement
                .on(startVideoEvent, onStartVideoElement)
                .on(stopVideoEvent, onEndVideoElement)
                .on(leaveEvent, onLeaveVideoElement);
        }

		function removeListeners( videoElement ) {

			videoElement
				.off(startVideoEvent, onStartVideoElement)
				.off(stopVideoEvent, onEndVideoElement)
				.off(leaveEvent, onLeaveVideoElement);

			if (isIPad || isIPhone) {
				jq(document.body)
					.off("touchend", onTouchendDocument)
					.off("touchstart", onTouchstartDocument);
			}

			if (canFireTouchmove) {
				videoElement.off("touchmove", preventVerticalScrolling);
			}
		}

		function onStartVideoElement(e) {
			lockNavigation();

			videoElement.focus();

			if (isOnShadowedElement(e).onWebkitMedia && !(isIPad || isIPhone)) {
				return cancelEvent(e);
			}
		}

		function onEndVideoElement() {
			unlockNavigation();
			switchPlayMode();
		}

		function onLeaveVideoElement() {
			unlockNavigation();
		}

		function onTouchstartDocument() {
			unlockNavigation();
		}

		function onTouchendDocument() {
			unlockNavigation();
		}

		// revealing public API
		return {
			onLoading: function () {

				if (!prepareForAuthoringMode()) {
                    context.stage.event.bindOnAudioControlChanged(this);
					context.page.event.bindOnEnter(this);
					context.page.event.bindOnExit(this);
				}
			},

			onEnterPage: function () {

                createVideo();

				if (isIPad || isIPhone) {
					jq(document.body)
						.on("touchstart", onTouchstartDocument)
						.on("touchend", onTouchendDocument);
				}
			},

			onExitPage: function () {
				removeListeners( videoElement );
				removeVideo();
			},

            onAudioControlChanged: function() {
				setAudioState();
			}
		};
	};
});